from plone.indexer import indexer

from Products.Archetypes.interfaces import IBaseObject

from collective.contentfiles2aws.awsfile import AWSFile
from collective.contentfiles2aws.interfaces import IAWSField
from collective.contentfiles2aws.interfaces import IAWSImageField

@indexer(IBaseObject)
def aws_sources(obj):
    sids = {}
    if not hasattr(obj, 'schema'):
        return

    obj_fields = obj.schema.fields()
    for f in obj_fields:
        if IAWSField.providedBy(f):
            accessor= f.getAccessor(obj)
            field_content = accessor()
            if not isinstance(field_content, AWSFile):
                return

            fname = f.getName()
            sids[fname] = field_content.source_id
            if IAWSImageField.providedBy(f):
                for n in f.getAvailableSizes(obj).keys():
                    scale = f.getScale(obj, scale=n)
                    if isinstance(scale, AWSFile):
                      scale_name = '%s_%s' % (fname, n)
                      sids[scale_name] = scale.source_id
    if sids:
        return sids



from Products.CMFCore.utils import getToolByName

from collective.contentfiles2aws.config import PROJECTNAME


DEFAULT_PROFILE = 'profile-%s:default' % PROJECTNAME

def upgrade_step(upgrade_product, version): 
    """Decorator for updating the QuickInstaller of a upgrade"""
    def wrap_func(fn):
        def wrap_func_args(context,*args, **kw):
            p = getToolByName(context, 'portal_quickinstaller'
                ).get(upgrade_product)
            print "%s: Upgrading to %s" % (upgrade_product, version)
            setattr(p, 'installedversion', version)
            return fn(context,*args, **kw)
        return wrap_func_args
    return wrap_func


@upgrade_step(PROJECTNAME, '1.0')
def upgrade_to_1_0(context):
    context.runImportStepFromProfile(DEFAULT_PROFILE, 'catalog')



from zope.i18nmessageid import MessageFactory

from Products.CMFCore.utils import ContentInit
from Products.Archetypes import atapi

from collective.contentfiles2aws.content import AWSFile
from collective.contentfiles2aws.content import AWSImage
from collective.contentfiles2aws import config
from collective.contentfiles2aws import content_types_patch

MFactory = MessageFactory(config.PROJECTNAME)


def initialize(context):

    content_types, constructors, ftis = atapi.process_types(
        atapi.listTypes(config.PROJECTNAME),
        config.PROJECTNAME)

    for atype, constructor in zip(content_types, constructors):
        ContentInit('%s: %s' % (config.PROJECTNAME, atype.portal_type),
            content_types      = (atype,),
            permission         = config.ADD_PERMISSIONS[atype.portal_type],
            extra_constructors = (constructor,),
            ).initialize(context)



import os
import unittest2

from OFS.Image import File
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage

from plone.app.testing import setRoles, TEST_USER_ID

from collective.contentfiles2aws.awsfile import AWSFile
from collective.contentfiles2aws.testing import \
    AWS_CONTENT_FILES_INTEGRATION_TESTING


class AWSFileTestCase(unittest2.TestCase):
    """ AWSFile test case. """

    layer =  AWS_CONTENT_FILES_INTEGRATION_TESTING

    def _get_image(self, filename='image.gif'):
        dir_name = os.path.dirname(os.path.abspath(__file__))
        return open('%s/data/%s' % (dir_name, filename), 'rb')

    def setUp(self):
        self.portal = self.layer['portal']
        self.conf_sheet=self.portal.portal_properties.contentfiles2aws

    def test_AWSFileFTI(self):
        # test some basic fti properties.
        typestool = getToolByName(self.portal, 'portal_types')
        info = typestool._getOb('AWSFile')

        title = info.getProperty('title')
        self.assertEqual('AWSFile', title)

        meta_type = info.getProperty('content_meta_type')
        self.assertEqual('AWSFile', meta_type)

        global_allow = info.getProperty('global_allow')
        self.assert_(global_allow)

        view_methods = info.getProperty('view_methods')
        self.assert_('file_view' in view_methods)

    def test_AWSFileCreation(self):
        self.conf_sheet._updateProperty('AWS_BUCKET_NAME', 'contentfiles')
        self.conf_sheet._updateProperty('USE_AWS', True)

        fid = self.portal.invokeFactory('AWSFile', 'aws_file')
        aws_file = getattr(self.portal, fid)
        aws_file.update(file=self._get_image())
        self.assertEqual(aws_file.portal_type, 'AWSFile')

        self.assert_(aws_file.schema['file'].url(
            aws_file).startswith('http://contentfiles.s3.amazonaws.com/'))

    def test_AWSFileUpdate(self):
        self.conf_sheet._updateProperty('AWS_BUCKET_NAME', 'contentfiles')
        self.conf_sheet._updateProperty('USE_AWS', True)

        fid = self.portal.invokeFactory('AWSFile', 'aws_file')
        aws_file = getattr(self.portal, fid)
        aws_file.update(file=self._get_image())

        # check if data attribute is empty
        self.assert_(aws_file.getField('file').get(aws_file).data)

        # update aws file with new file
        aws_file.update(file=self._get_image(filename='aws.gif'))

        # check if data attribute is empty
        value = aws_file.getField('file').get(aws_file)
        self.assert_(not value.__dict__['data'])

        # update file when aws is turned off
        self.conf_sheet._updateProperty('USE_AWS', False)
        aws_file.update(file=self._get_image())

        # check if file field attribute value has proper type (File)
        value = aws_file.getField('file').get(aws_file)
        self.assert_(isinstance(value, File))

        # check if data attribute is not empty
        self.assert_(value.data)

        # update file when aws is turned on
        self.conf_sheet._updateProperty('USE_AWS', True)
        aws_file.update(file=self._get_image(filename='aws.gif'))

        # check if file field attribute value has proper type (AWSFile)
        value = aws_file.getField('file').get(aws_file)
        self.assert_(isinstance(value, AWSFile))
        self.assert_(not 'data' in value.__dict__)

        # update file when there is problems with amazon
        self.conf_sheet._updateProperty('AWS_KEY_ID', 'BAD_KEY')
        aws_file.update(file=self._get_image())

        # check if file field attribute value has proper type (File)
        value = aws_file.getField('file').get(aws_file)
        self.assert_(isinstance(value, File))
        self.assert_(value.data)

    def test_AWSFileCopy(self):
        self.conf_sheet._updateProperty('AWS_BUCKET_NAME', 'contentfiles')
        self.conf_sheet._updateProperty('USE_AWS', True)

        fid = self.portal.invokeFactory('AWSFile', 'aws_file')
        aws_file = getattr(self.portal, fid)
        aws_file.update(file=self._get_image())

        cp = self.portal.manage_copyObjects(ids=['aws_file'])

        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.manage_pasteObjects(cb_copy_data=cp)
        setRoles(self.portal, TEST_USER_ID, ["Member"])

        original_source_id = aws_file.getField('file').get(aws_file).source_id

        aws_file_copy = getattr(self.portal, 'copy_of_aws_file')
        copy_source_id = \
                aws_file_copy.getField('file').get(aws_file_copy).source_id

        self.assertNotEqual(original_source_id, copy_source_id)

        # check that after original file remove we still have data in
        # cloned object.
        self.portal.manage_delObjects(ids=['aws_file'])
        self.assert_(aws_file_copy.getField('file').get(aws_file_copy).data)

        # check file copy handler when 'aws storage' is turned off.
        self.conf_sheet._updateProperty('USE_AWS', False)

        # lets make new copy from previous one.
        cp = self.portal.manage_copyObjects(ids=['copy_of_aws_file'])

        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.manage_pasteObjects(cb_copy_data=cp)
        setRoles(self.portal, TEST_USER_ID, ["Member"])

        # ensure that copy object was successfuly removed.
        self.assert_('copy_2_of_aws_file' not in self.portal.objectIds())

        messages = IStatusMessage(self.layer['request']).showStatusMessages()
        error_message = ('Could not copy remote source. To be able to copy '
                        'object properly, please activate AWS storage')
        self.assert_(error_message in [m.message for m in messages])

        # check that folderish objects with aws files inside works properly.
        self.conf_sheet._updateProperty('USE_AWS', True)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory('Folder', 'folder')
        folder = getattr(self.portal, 'folder')
        fid = folder.invokeFactory('AWSFile', 'aws_file')
        folder.aws_file.update(file=self._get_image())
        cp = self.portal.manage_copyObjects(ids=['folder'])

        self.portal.manage_pasteObjects(cb_copy_data=cp)

        folder_copy = getattr(self.portal, 'copy_of_folder')
        self.assert_('aws_file' in folder_copy.objectIds())

        aws_file = folder.aws_file
        aws_file_copy = folder_copy.aws_file

        original_source_id = aws_file.getField('file').get(aws_file).source_id
        copy_source_id = \
                aws_file_copy.getField('file').get(aws_file_copy).source_id

        self.assertNotEqual(original_source_id, copy_source_id)

        # check file copy handler for folderish objects
        # when 'aws storage' is turned off.
        self.conf_sheet._updateProperty('USE_AWS', False)

        cp = self.portal.manage_copyObjects(ids=['folder'])

        folder.manage_pasteObjects(cb_copy_data=cp)

        # ensure that copy object was successfuly removed.
        self.assert_('copy2_of_folder' not in folder.objectIds())

        messages = IStatusMessage(self.layer['request']).showStatusMessages()
        error_message = ('Could not copy remote source. To be able to copy '
                        'object properly, please activate AWS storage')
        self.assert_(error_message in [m.message for m in messages])

        setRoles(self.portal, TEST_USER_ID, ["Member"])


def test_suite():
    suite = unittest2.TestSuite()
    suite.addTest(unittest2.makeSuite(AWSFileTestCase))
    return suite



import os
import unittest2

from OFS.Image import File
from zope.component import getUtility

from collective.contentfiles2aws.awsfile import AWSFile
from collective.contentfiles2aws.interfaces import IAWSFileClientUtility
from collective.contentfiles2aws.testing import \
    AWS_CONTENT_FILES_INTEGRATION_TESTING


class AWSStorageTestCase(unittest2.TestCase):
    """ AWS File test case. """

    layer =  AWS_CONTENT_FILES_INTEGRATION_TESTING

    def _get_image(self):
        dir_name = os.path.dirname(os.path.abspath(__file__))
        return open('%s/data/image.gif' % dir_name, 'rb')

    def setUp(self):
        self.portal = self.layer['portal']
        # set test bucket name
        self.sheet = self.portal.portal_properties.contentfiles2aws
        self.sheet._updateProperty('USE_AWS', True)
        self.sheet._updateProperty('AWS_BUCKET_NAME', 'contentfiles')

        id = self.portal.invokeFactory('AWSFile', 'awsfile')
        self.awsfile = getattr(self.portal, id)
        self.awsfile.update(file=self._get_image())
        self.storage = self.awsfile.schema['file'].storage
        self.aws_file = self.awsfile.getFile()

    def test_make_prefix(self):
        """ Tests generated prefix creation. """
        prefix = self.storage.make_prefix()
        # each method call should generate new hash
        self.assertNotEqual(prefix, self.storage.make_prefix())

    def test_NormalizedName(self):
        """ Test filename normalizer. """

        filename = u'\u0456\u043c\u044f\u0444\u0430\u0439\u043b\u0443.txt'
        self.assertEqual('45643c44f44443043b443.txt',
                         self.storage.getNormalizedName(filename))

    def test_sourceId(self):
        """ Tests source id generation."""

        source_id = self.storage.getSourceId('file',
                                             'filename.gif',
                                             self.awsfile)
        self.assert_(source_id.startswith(self.aws_file.UID()))
        self.assert_(source_id.endswith('_file_filename.gif'))

    def test_update_source(self):
        """ Test update_source method."""
        self.storage.update_source(self.aws_file, 'new text', self.awsfile,
                                   'newfilename.gif', 'image/gif', width='100',
                                   height='150')

        aws_utility = getUtility(IAWSFileClientUtility)
        as3client = aws_utility.getFileClient()
        self.assertEqual(as3client.get(self.aws_file.source_id), 'new text')
        self.assert_(self.aws_file.filename, 'newfilename.gif')
        self.assert_(self.aws_file.content_type, 'image/gif')

    def test_do_migrate(self):
        self.sheet._updateProperty('USE_AWS', False)

        id = self.portal.invokeFactory('AWSFile', 'file')
        file_ = getattr(self.portal, id)
        file_.update(file=self._get_image())
        storage = file_.schema['file'].storage
        file__ = file_.getFile()
        self.assert_(isinstance(storage._do_migrate(file__, file_), AWSFile))

    def test_get(self):
        self.assertEqual(self.aws_file, self.storage.get('file', self.awsfile))

        self.sheet._updateProperty('USE_AWS', False)
        id = self.portal.invokeFactory('AWSFile', 'file')
        file_ = getattr(self.portal, id)
        file_.update(file=self._get_image())
        storage = file_.schema['file'].storage
        file__ = file_.getFile()

        self.assert_(isinstance(file__, File))

        self.sheet._updateProperty('USE_AWS', True)
        file_.REQUEST['file_migrate'] = True
        self.assert_(isinstance(storage.get('file', file_), AWSFile))

    def test_set(self):
        self.sheet._updateProperty('USE_AWS', False)
        id = self.portal.invokeFactory('AWSFile', 'file')
        file_ = getattr(self.portal, id)
        storage = file_.schema['file'].storage

        ofsfile = File(id, '', self._get_image(), content_type='image/jpeg')
        setattr(ofsfile, 'filename', 'image.jpg')
        storage.set('file', file_, ofsfile)
        self.assert_(storage.get('file', file_), File)

        self.sheet._updateProperty('USE_AWS', True)
        self.assert_(storage.get('file', file_), AWSFile)

    def test_unset(self):
        id = self.portal.invokeFactory('AWSFile', 'file')
        file_ = getattr(self.portal, id)
        storage = file_.schema['file'].storage

        ofsfile = File(id, '', self._get_image(), content_type='image/jpeg')
        setattr(ofsfile, 'filename', 'image.jpg')
        storage.set('file', file_, ofsfile)
        self.assert_(storage.get('file', file_), AWSFile)

        storage.unset('file', file_)
        self.assertRaises(AttributeError, storage.get, 'file', file_)

def test_suite():
    suite = unittest2.TestSuite()
    suite.addTest(unittest2.makeSuite(AWSStorageTestCase))
    return suite



from zope.interface import implements
from zope.app.component.hooks import getSite

from Products.CMFCore.utils import getToolByName

from collective.contentfiles2aws.client import AWSFileClient
from collective.contentfiles2aws.interfaces import IAWSFileClientUtility
from collective.contentfiles2aws.config import AWSCONF_SHEET


class AWSFileClientUtility(object):
    """
    """
    implements(IAWSFileClientUtility)

    def active(self):
        pp = getToolByName(getSite(), 'portal_properties')
        awsconf_sheet = getattr(pp, AWSCONF_SHEET)
        return awsconf_sheet.getProperty('USE_AWS')

    def getAWSConfiguration(self):
        """ Collect configuration infomation for aws client. """
        #TODO: temporary we will save configuration in property sheet.
        #      it will be good to have more flexible solution for this.
        pp = getToolByName(getSite(), 'portal_properties')
        awsconf_sheet = getattr(pp, AWSCONF_SHEET)
        aws_key_id = awsconf_sheet.getProperty('AWS_KEY_ID')
        aws_seecret_key = awsconf_sheet.getProperty('AWS_SEECRET_KEY')
        aws_bucket_name = awsconf_sheet.getProperty('AWS_BUCKET_NAME')
        aws_filename_prefix = awsconf_sheet.getProperty('AWS_FILENAME_PREFIX')

        return {'aws_key_id': aws_key_id,
                'aws_seecret_key': aws_seecret_key,
                'aws_bucket_name': aws_bucket_name,
                'aws_filename_prefix': aws_filename_prefix}

    def getBucketName(self):
        return self.getAWSConfiguration()['aws_bucket_name']

    def getAWSFilenamePrefix(self):
        return self.getAWSConfiguration()['aws_filename_prefix']

    def getFileClient(self):
        """ Provide an aws file client. """
        config = self.getAWSConfiguration()
        client = AWSFileClient(config['aws_key_id'],
                               config['aws_seecret_key'],
                               config['aws_bucket_name'],
                               aws_filename_prefix=
                               config['aws_filename_prefix'])
        return client

aws_utility = AWSFileClientUtility()



import os
import unittest2

from collective.contentfiles2aws.testing import \
    AWS_CONTENT_FILES_INTEGRATION_TESTING


class AWSFileFieldTestCase(unittest2.TestCase):
    """
    """
    layer = AWS_CONTENT_FILES_INTEGRATION_TESTING

    def _get_image(self):
        dir_name = os.path.dirname(os.path.abspath(__file__))
        return open('%s/data/image.gif' % dir_name, 'rb')

    def setUp(self):
        portal = self.layer['portal']
        fid = portal.invokeFactory('AWSFile', 'awsfile')
        self.awsfile = getattr(portal, fid)
        self.awsfile.update(file=self._get_image())

    def test_use_aws(self):
        self.assert_(not self.awsfile.schema['file'].use_aws(self.awsfile))

        portal = self.layer['portal']
        sheet = portal.portal_properties.contentfiles2aws
        sheet._updateProperty('USE_AWS', True)

        self.assert_(self.awsfile.schema['file'].use_aws(self.awsfile))

    def test_migrate(self):
        portal = self.layer['portal']
        sheet = portal.portal_properties.contentfiles2aws
        sheet._updateProperty('USE_AWS', True)

        self.assert_(self.awsfile.schema['file'].migrate(self.awsfile))

        fid = portal.invokeFactory('AWSFile', 'awsfile2')
        awsfile2 = getattr(portal, fid)
        awsfile2.update(file=self._get_image())

        self.assert_(not awsfile2.schema['file'].migrate(awsfile2))

    def test_url(self):
        portal = self.layer['portal']
        sheet = portal.portal_properties.contentfiles2aws
        sheet._updateProperty('AWS_BUCKET_NAME', 'contentfiles')
        sheet._updateProperty('USE_AWS', True)
        fid = portal.invokeFactory('AWSFile', 'awsfile2')
        awsfile2 = getattr(portal, fid)
        awsfile2.update(file=self._get_image())

        self.assert_(awsfile2.schema['file'].url(
            awsfile2).startswith('http://contentfiles.s3.amazonaws.com/'))


def test_suite():
    suite = unittest2.TestSuite()
    suite.addTest(unittest2.makeSuite(AWSFileFieldTestCase))
    return suite



import transaction
from zope.component import getUtility, queryMultiAdapter
from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.ArchetypeTool import getType

from plone.indexer.interfaces import IIndexableObject

from collective.contentfiles2aws.interfaces import IAWSFileClientUtility
from collective.contentfiles2aws.interfaces import IAWSField


class ContentMigrationView(BrowserView):
    """ Simple browser view for content migration. """

    bsize = 100

    def get_types(self):
        types_ = []
        pt = getToolByName(self.context, 'portal_types')
        for type_ in pt.listTypeInfo():
            meta_type = type_.Metatype()
            product = type_.product
            try:
                t = getType(meta_type, product)
            except KeyError:
                continue
            if 'schema' in t:
                for f in t['schema'].fields():
                    if IAWSField.providedBy(f):
                        tid = type_.id
                        if tid not in types_:
                            types_.append(tid)
        return types_

    def get_bsize(self):
        bsize = self.request.get('bsize', 0)
        if bsize:
            return int(bsize)
        return self.bsize

    def _update_metadata(self, catalog, obj):
        uid = '/'.join(obj.getPhysicalPath())
        wrapper = queryMultiAdapter((obj, catalog), IIndexableObject)
        catalog._catalog.updateMetadata(wrapper, uid)

    def _migrate(self, query, types):
        count = 0
        results = {}
        pc = getToolByName(self.context, 'portal_catalog')

        for ptype in types:
            query['portal_type'] = ptype
            results[ptype] = 0
            for b in pc(**query):
                obj = b.getObject()
                for f in obj.schema.fields():
                    if IAWSField.providedBy(f):
                        name = f.getName()
                        migrate = '%s_migrate' % name
                        obj.REQUEST[migrate] = True
                        f.get(obj)
                        self._update_metadata(pc, obj)
                        count = count + 1
                        if count % self.get_bsize() == 0:
                            transaction.savepoint(optimistic=True)
                        results[ptype] = results[ptype] + 1
        return results

    def __call__(self):
        aws_utility = getUtility(IAWSFileClientUtility)
        if not aws_utility.active():
            return "AWSStorage isn't active. Nothing to do!"

        result = ''
        pc = getToolByName(self.context, 'portal_catalog')
        query = {'path': {'query': '/'.join(self.context.getPhysicalPath())}}
        content_type = self.request.get('content_type', '')
        migrated = None
        if content_type:
            if content_type.lower() == 'all':
                migrated = self._migrate(query, self.get_types())
            else:
                migrated = self._migrate(query, (content_type,))

            if migrated:
                result = '\n'.join(['%s: %d fields migrated' % (k, v)
                                    for k, v in migrated.items()])
        else:
            for ptype in self.get_types():
                query['portal_type'] = ptype
                result += "%s: %d\n" % (ptype, len(pc(**query)))

        return result



import os
import unittest2

from Products.CMFCore.utils import getToolByName

from collective.contentfiles2aws.testing import \
    AWS_CONTENT_FILES_INTEGRATION_TESTING


class AWSImageTestCase(unittest2.TestCase):
    """ AWSFile test case. """

    layer =  AWS_CONTENT_FILES_INTEGRATION_TESTING

    def _get_image(self):
        dir_name = os.path.dirname(os.path.abspath(__file__))
        return open('%s/data/image.gif' % dir_name, 'rb')

    def setUp(self):
        self.portal = self.layer['portal']
        self.conf_sheet=self.portal.portal_properties.contentfiles2aws

    def test_AWSImageFTI(self):
        # test some basic fti properties.
        typestool = getToolByName(self.layer['portal'], 'portal_types')
        info = typestool._getOb('AWSImage')

        title = info.getProperty('title')
        self.assertEqual('AWSImage', title)

        meta_type = info.getProperty('content_meta_type')
        self.assertEqual('AWSImage', meta_type)

        global_allow = info.getProperty('global_allow')
        self.assert_(global_allow)

        view_methods = info.getProperty('view_methods')
        self.assert_('image_view' in view_methods)

    def test_AWSImageCreation(self):
        self.conf_sheet._updateProperty('AWS_BUCKET_NAME', 'contentfiles')
        self.conf_sheet._updateProperty('USE_AWS', True)

        fid = self.portal.invokeFactory('AWSImage', 'aws_image')
        aws_image = getattr(self.portal, fid)
        aws_image.update(image=self._get_image())
        self.assertEqual(aws_image.portal_type, 'AWSImage')

        image_field = aws_image.schema['image']
        self.assert_('http://contentfiles.s3.amazonaws.com/' in \
                image_field.tag(aws_image))


def test_suite():
    suite = unittest2.TestSuite()
    suite.addTest(unittest2.makeSuite(AWSImageTestCase))
    return suite



class FileClientError(Exception):

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)

class FileClientRetrieveError(FileClientError):
    """ Base Retrieve Error Exception"""
    pass


class FileClientStoreError(FileClientError):
    """ Base Store Error Exception """
    pass


class FileClientRemoveError(FileClientError):
    """ Base Remove Error Exception """
    pass

class FileClientCopyError(FileClientError):
    """ Base Copy Error Exception """
    pass


class FSFileClient(object):
    pass



from cgi import escape

from AccessControl import ClassSecurityInfo
from OFS.Image import Image

from zope.interface import implements

from Products.Archetypes.Field import FileField, ImageField
from Products.CMFCore import permissions
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.debug import log
from Products.Archetypes.Registry import registerField

from collective.contentfiles2aws.awsfile import AWSFile
from collective.contentfiles2aws.widgets import AWSFileWidget
from collective.contentfiles2aws.widgets import AWSImageWidget
from collective.contentfiles2aws.interfaces import IAWSFileField
from collective.contentfiles2aws.interfaces import IAWSImageField
from collective.contentfiles2aws.config import AWSCONF_SHEET

_marker = []

try:
    import PIL.Image
except ImportError:
    # no PIL, no scaled versions!
    log("Warning: no Python Imaging Libraries (PIL) found." +
        "Archetypes based ImageField's don't scale if neccessary.")
    HAS_PIL = False
    PIL_ALGO = None
else:
    HAS_PIL = True
    PIL_ALGO = PIL.Image.ANTIALIAS


class AWSFileField(FileField):
    """Something that may be a file, but is not an image and doesn't
    want text format conversion"""

    implements(IAWSFileField)

    _properties = FileField._properties.copy()
    _properties.update({'widget': AWSFileWidget})

    security = ClassSecurityInfo()

    def use_aws(self, instance):
        pp = getToolByName(instance, 'portal_properties')
        awsconf_sheet = getattr(pp, AWSCONF_SHEET)
        return awsconf_sheet.getProperty('USE_AWS')

    def migrate(self, instance):
        if not self.use_aws(instance):
            return

        obj = self.get(instance)
        if isinstance(obj, AWSFile) or not obj:
            return False
        else:
            return True

    def url(self, instance):
        fobj = self.get(instance, raw=True, unwrapped=True)

        if isinstance(fobj, AWSFile):
            return fobj.absolute_url()

        return '%s/at_download/%s' % (instance.absolute_url(),
                                      self.getName())

registerField(AWSFileField,
              title='AWS File',
              description='Used for storing files in amazon s3 service.')


class AWSImageField(ImageField, AWSFileField):

    implements(IAWSImageField)

    _properties = ImageField._properties.copy()
    _properties.update({
        'widget': AWSImageWidget,
        'content_class': Image,
        'fallback_content_class': AWSFile})

    security = ClassSecurityInfo()

    default_view = "view"

    def _wrapValue(self, instance, value, **kwargs):
        """Wraps the value in the content class if it's not wrapped
        """
        if isinstance(value, self.content_class) or \
                isinstance(value, self.fallback_content_class):
            return value
        mimetype = kwargs.get('mimetype', self.default_content_type)
        filename = kwargs.get('filename', '')
        obj = self._make_file(self.getName(), title='',
                              file=value, instance=instance)
        setattr(obj, 'filename', filename)
        setattr(obj, 'content_type', mimetype)
        try:
            delattr(obj, 'title')
        except (KeyError, AttributeError):
            pass

        return obj

    security.declarePrivate('get')
    def get(self, instance, **kwargs):
        if not self.use_aws(instance):
            return ImageField.get(self, instance, **kwargs)

        request = instance.REQUEST
        if not isinstance(request, type('')) and \
                instance.REQUEST.get('%s_migrate' % self.getName(), ''):
            # migrate scales
            kwargs['migrate'] = True
            for n in self.getAvailableSizes(instance).keys():
                self.getScale(instance, scale=n, **kwargs)
        return ImageField.get(self, instance, **kwargs)

    security.declareProtected(permissions.View, 'tag')
    def tag(self, instance, scale=None, height=None, width=None, alt=None,
            css_class=None, title=None, **kwargs):
        """Create a tag including scale
        """

        image = self.getScale(instance, scale=scale)
        if not self.use_aws(instance) and not isinstance(image, AWSFile):
            return ImageField.tag(self, instance, scale=scale, height=height,
                                  width=width, alt=alt, css_class=css_class,
                                  title=title, **kwargs)
        if image:
            img_width, img_height = self.getSize(instance, scale=scale)
            url = image.absolute_url()
        else:
            img_height = 0
            img_width = 0
            url = instance.absolute_url()

        if height is None:
            height = img_height
        if width is None:
            width = img_width

        values = {'src': url,
                  'alt': escape(alt and alt or instance.Title(), 1),
                  'title': escape(title and title or instance.Title(), 1),
                  'height': height,
                  'width': width}

        result = '<img src="%(src)s" alt="%(alt)s" title="%(title)s" '\
                 'height="%(height)s" width="%(width)s"' % values

        if css_class is not None:
            result = '%s class="%s"' % (result, css_class)

        for key, value in kwargs.items():
            if value:
                result = '%s %s="%s"' % (result, key, value)

        return '%s />' % result

registerField(AWSImageField,
              title='AWS Image',
              description=('Used for storing images. '
                           'Images can then be retrieved in '
                           'different thumbnail sizes'))



import urllib
import logging
from boto.exception import S3CopyError
from boto.exception import S3ResponseError
from boto.s3.connection import S3Connection

from zope.interface import implements

from collective.contentfiles2aws.client.interfaces import IAWSFileClient
from collective.contentfiles2aws.client.fsclient import \
    FileClientRetrieveError, FileClientStoreError, FileClientRemoveError
from collective.contentfiles2aws.client.fsclient import FileClientCopyError

logger = logging.getLogger(__name__)


class AWSFileClientStoreError(FileClientStoreError):
    pass


class AWSFileClientRetrieveError(FileClientRetrieveError):
    pass


class AWSFileClientRemoveError(FileClientRemoveError):
    pass


class AWSFileClientCopyError(FileClientCopyError):
    pass


class AWSFileClient(object):
    """ Simple amazon s3 file client. """

    implements(IAWSFileClient)

    def __init__(self, aws_key_id, aws_seecret_key, bucket_name,
                 aws_filename_prefix=None):
        self._aws_key_id = aws_key_id
        self._aws_seecret_key = aws_seecret_key
        self.bucket_name = bucket_name
        self.connection = S3Connection(aws_key_id, aws_seecret_key)
        self.aws_filename_prefix = aws_filename_prefix

    def _get_bucket_name(self, **kw):
        bucket_name = self.bucket_name
        if hasattr(kw, 'bucket_name') and kw['bucket_name']:
            bucket_name = kw['bucket_name']
        return bucket_name

    def _get_key(self, filename):
        if not self.aws_filename_prefix:
            return filename
        return '%s/%s' % (self.aws_filename_prefix, filename)

    def get(self, filename, **kw):
        """ Get file with specified filename from amazon.

        Search file on amazon storage and if file with specified name
        exists returns file content.

        :param filename: name of file.
        :type filename: string
        :returns: file content in case file exists,
        ohterwise returns None

        """

        bucket_name = self._get_bucket_name(**kw)

        try:
            bucket = self.connection.get_bucket(bucket_name)
            key = bucket.get_key(self._get_key(filename))
            if key:
                return key.get_contents_as_string()
        except S3ResponseError, e:
            logger.exception('%s, %s, %s' % (e.status, e.reason, e.message))
            raise AWSFileClientRetrieveError(e.message)

    def put(self, filename, data, **kw):
        """Writes file to amazon storage under provided file name.

        :param filename: name of file.
        :type filename: string
        :param data: file content.
        :type: string: string
        :param kw: dictionary with file metadata
        :type kw: dict
        :returns: None

        """

        mimetype = None
        if 'mimetype' in kw and kw['mimetype']:
            mimetype = kw['mimetype']

        conn = self.connection
        bucket_name = self._get_bucket_name(**kw)
        try:
            if bucket_name in [b.name for b in conn.get_all_buckets()]:
                bucket = conn.get_bucket(bucket_name)
            else:
                bucket = conn.create_bucket(bucket_name)

            key = bucket.get_key(self._get_key(filename))
            if not key:
                key = bucket.new_key(self._get_key(filename))
            if mimetype:
                key.set_metadata('Content-Type', mimetype)
                mtype, msubtype = mimetype.split('/')
                if mtype != 'image':
                    # set content disposition metadata with original
                    # filename for files.
                    if 'original_name' in kw and kw['original_name']:
                        fname = urllib.quote(kw['original_name'])
                    else:
                        fname = '_'.join(filename.split('_')[3:])
                    key.set_metadata("Content-Disposition",
                                     "attachment; filename*=UTF-8''%s" % fname)
            key.set_contents_from_string(data)
            key.set_acl('public-read')
        except S3ResponseError, e:
            logger.exception('%s, %s, %s' % (e.status, e.reason, e.message))
            raise AWSFileClientStoreError(e.message)

    def delete(self, filename, **kw):
        """ Removes file form amazon storage using provided filename.

        :param filename: name of file.
        :type filename: string

        """

        bucket_name = self._get_bucket_name(**kw)
        try:
            bucket = self.connection.get_bucket(bucket_name)
            bucket.delete_key(self._get_key(filename))
        except S3ResponseError, e:
            logger.exception('%s, %s, %s' % (e.status, e.reason, e.message))
            raise AWSFileClientRemoveError(u"Couldn't delete %s file. %s" %
                                           (filename.decode('utf-8'),
                                            e.message and
                                            e.message.decode('utf-8') or ''))

    def source_url(self, filename, **kw):
        bucket_name = self._get_bucket_name(**kw)
        return "http://%s.%s/%s" % (bucket_name,
                                    self.connection.server,
                                    self._get_key(filename))

    def copy_source(self, filename, new_filename, **kw):
        """ Create a copy of specified file.

        :param filename: source file name
        :type filename: string
        :param new_filename: new file name
        :type new_filename: string

        """

        bucket_name = self._get_bucket_name(**kw)

        metadata = None
        if 'metadata' in kw:
            metadata = kw['metadata']

        try:
            bucket = self.connection.get_bucket(bucket_name)
            bucket.copy_key(self._get_key(new_filename), bucket_name,
                            self._get_key(filename), metadata=metadata)

            # copy permissions
            origin_key = bucket.get_key(self._get_key(filename))
            new_key = bucket.get_key(self._get_key(new_filename))
            new_key.set_acl(origin_key.get_acl())
        except (S3CopyError, S3ResponseError), e:
            logger.exception('%s, %s, %s' % (e.status, e.reason, e.message))
            raise AWSFileClientCopyError(
                u"Couldn't make copy for  %s file. %s" %
                (filename.decode('utf-8'), e.message.decode('utf-8')))



from collective.contentfiles2aws.client.interfaces.fsclient import IFileClient


class IAWSFileClient(IFileClient):
    """ File client for amazon s3 file storage.

    Here is described aws specific methods. For more general
    methods please see base interface.

    """

    def source_url(filename, **kw):
        """ Source url constructor.

        Build and return url to file source.

        """



from Products.Archetypes.atapi import FileWidget
from Products.Archetypes.atapi import ImageWidget
from Products.Archetypes.Registry import registerWidget

class AWSFileWidget(FileWidget):
    _properties = FileWidget._properties.copy()
    _properties.update({'macro' : "awsfile"})

    def href(self, instance, field):
        return field.url(instance)

registerWidget(AWSFileWidget,
               title='AWS File',
               description='Renders a HTML widget upload a file',
               used_for=('collective.contentfiles2aws.fields.AWSFileField',)
               )

class AWSImageWidget(ImageWidget):
    _properties = ImageWidget._properties.copy()
    _properties.update({'macro' : "awsimage"})

registerWidget(AWSImageWidget,
               title='AWS Image',
               description=('Renders a HTML widget for '
                            'uploading/displaying an image'),
               used_for=('collective.contentfiles2aws.fields.AWSImageField',)
               )



from cgi import escape

from OFS.Image import File
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import view as View

from zope.component import getUtility

from collective.contentfiles2aws.interfaces import IAWSFileClientUtility
from collective.contentfiles2aws.client.fsclient import FileClientRetrieveError


class AWSFile(File):
    """ Oject that represents AWS file."""

    meta_type = 'AWS File'
    filename = u''
    data = ''
    # for image files
    height=''
    width=''

    security = ClassSecurityInfo()

    def __init__(self, id, size=0,filename=u'', content_type=''):
        self.__name__ = id
        self.size = size
        self.filename = filename
        self.content_type = content_type
        self.source_id = None

    def get_data(self):
        """ Retreives source from object.

        First of all method looks into object 'data' attribute and if
        it is not empty return the value. If 'data' attribute is
        empty method will try to load data from Amazon storage.

        """

        if 'data' in self.__dict__ and self.__dict__['data']:
            return self.__dict__['data']

        aws_utility = getUtility(IAWSFileClientUtility)
        as3client = aws_utility.getFileClient()
        if self.source_id:
            try:
                return as3client.get(self.source_id)
            except FileClientRetrieveError:
                return ''

    def set_data(self, value):
        """ Temporary set file data to 'data' attribute.

        AWS storage will migrate data to Amazon and clean up
        'data' on save action.
        """
        self.__dict__['data'] = value

    data = property(get_data, set_data)

    security.declareProtected(View, 'index_html')
    def index_html(self, REQUEST, RESPONSE):
        """
        The default view of the contents of a File or Image.

        Returns the contents of the file or image.  Also, sets the
        Content-Type HTTP header to the objects content type.

        """
        return RESPONSE.redirect(self.absolute_url())

    def absolute_url(self):
        aws_utility = getUtility(IAWSFileClientUtility)
        as3client = aws_utility.getFileClient()
        return as3client.source_url(self.source_id)

    def remove_source(self):
        aws_utility = getUtility(IAWSFileClientUtility)
        as3client = aws_utility.getFileClient()
        if self.source_id:
            as3client.delete(self.source_id)

    security.declareProtected(View, 'tag')
    def tag(self, height=None, width=None, alt=None,
            scale=0, xscale=0, yscale=0, css_class=None, title=None, **args):

        if not (self.height or self.width):
            # this is file not image
            return ''

        if height is None: height=self.height
        if width is None:  width=self.width

        # Auto-scaling support
        xdelta = xscale or scale
        ydelta = yscale or scale

        if xdelta and width:
            width =  str(int(round(int(width) * xdelta)))
        if ydelta and height:
            height = str(int(round(int(height) * ydelta)))

        result='<img src="%s"' % (self.absolute_url())

        if alt is None:
            alt=getattr(self, 'alt', '')
        result = '%s alt="%s"' % (result, escape(alt, 1))

        if title is None:
            title=getattr(self, 'title', '')
        result = '%s title="%s"' % (result, escape(title, 1))

        if height:
            result = '%s height="%s"' % (result, height)

        if width:
            result = '%s width="%s"' % (result, width)

        if css_class is not None:
            result = '%s class="%s"' % (result, css_class)

        for key in args.keys():
            value = args.get(key)
            if value:
                result = '%s %s="%s"' % (result, key, value)

        return '%s />' % result



from Products.ATContentTypes.content.file import ATFile, ATFileSchema
from Products.ATContentTypes.content.image import ATImage, ATImageSchema
from Products.ATContentTypes.content.newsitem import ATNewsItem, \
    ATNewsItemSchema
from Products.ATContentTypes import ATCTMessageFactory as _
from Products.validation import V_REQUIRED
from Products.ATContentTypes.config import PROJECTNAME
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content.base import registerATCT

from collective.contentfiles2aws.fields import AWSFileField, AWSImageField
from collective.contentfiles2aws.widgets import AWSFileWidget, AWSImageWidget
from collective.contentfiles2aws.storage import AWSStorage


ATFileSchema = ATFileSchema.copy()
ATFileSchema['file'] = \
    AWSFileField('file',
              required=True,
              primary=True,
              searchable=True,
              languageIndependent=True,
              storage = AWSStorage(),
              validators = (('isNonEmptyFile', V_REQUIRED),
                             ('checkFileMaxSize', V_REQUIRED)),
              widget = AWSFileWidget(
                        description = '',
                        label=_(u'label_file', default=u'File'),
                        show_content_type = False,))

ATFile.schema = ATFileSchema
registerATCT(ATFile, PROJECTNAME)


ATImageSchema = ATImageSchema.copy()
ATImageSchema['image'] = \
    AWSImageField('image',
               required=True,
               primary=True,
               languageIndependent=True,
               storage = AWSStorage(),
               swallowResizeExceptions = zconf.swallowImageResizeExceptions.enable,
               pil_quality = zconf.pil_config.quality,
               pil_resize_algo = zconf.pil_config.resize_algo,
               max_size = zconf.ATImage.max_image_dimension,
               sizes= {'large'   : (768, 768),
                       'preview' : (400, 400),
                       'mini'    : (200, 200),
                       'thumb'   : (128, 128),
                       'tile'    :  (64, 64),
                       'icon'    :  (32, 32),
                       'listing' :  (16, 16),
                      },
               validators = (('isNonEmptyFile', V_REQUIRED),
                             ('checkImageMaxSize', V_REQUIRED)),
               widget = AWSImageWidget(
                        description = '',
                        label= _(u'label_image', default=u'Image'),
                        show_content_type = False,))

ATImage.schema = ATImageSchema
registerATCT(ATImage, PROJECTNAME)



ATNewsItemSchema = ATNewsItemSchema.copy()
ATNewsItemSchema['image'] = \
    AWSImageField('image',
           required = False,
           storage = AWSStorage(migrate=True),
           languageIndependent = True,
           max_size = zconf.ATNewsItem.max_image_dimension,
           sizes= {'large'   : (768, 768),
                   'preview' : (400, 400),
                   'mini'    : (200, 200),
                   'thumb'   : (128, 128),
                   'tile'    :  (64, 64),
                   'icon'    :  (32, 32),
                   'listing' :  (16, 16),
                  },
           validators = (('isNonEmptyFile', V_REQUIRED),
                                ('checkNewsImageMaxSize', V_REQUIRED)),
           widget = AWSImageWidget(
               description = _(u'help_news_image', default=u'Will be shown in the news listing, and in the news item itself. Image will be scaled to a sensible size.'),
               label= _(u'label_news_image', default=u'Image'),
               show_content_type = False)
           )

ATNewsItem.schema = ATNewsItemSchema
registerATCT(ATNewsItem, PROJECTNAME)



from zope.interface import Interface


class IFileClient(Interface):
    """ Simple file client."""

    def get(filename, **kw):
        """ Get file with specified filename.

        Search file in storage and if file with specified name
        exists returns file content.

        :param filename: name of file.
        :type filename: string
        :returns: file content in case file exists,
        ohterwise returns None

        """
    def put(filename, data, **kw):
        """Writes file to storage under provided file name.

        :param filename: name of file.
        :type filename: string
        :param mimetype: file mimetype.
        :type mimetype: string.
        :param data: file content.
        :type: string: string
        :returns: None

        """

    def delete(filename, **kw):
        """ Removes file form storage using provided filename.

        :param filename: name of file.
        :type filename: string

        """




import os
import unittest2

from zope.component import getUtility

from collective.contentfiles2aws.interfaces import IAWSFileClientUtility
from collective.contentfiles2aws.testing import \
    AWS_CONTENT_FILES_INTEGRATION_TESTING


class AWSFileTestCase(unittest2.TestCase):
    """ AWS File test case. """

    layer = AWS_CONTENT_FILES_INTEGRATION_TESTING

    def _get_image(self):
        dir_name = os.path.dirname(os.path.abspath(__file__))
        return open('%s/data/image.gif' % dir_name, 'rb')

    def setUp(self):
        portal = self.layer['portal']
        sheet = portal.portal_properties.contentfiles2aws
        sheet._updateProperty('USE_AWS', True)
        sheet._updateProperty('AWS_BUCKET_NAME', 'contentfiles')

        id = portal.invokeFactory('AWSFile', 'awsfile')
        self.awsfile = getattr(portal, id)
        self.awsfile.update(file=self._get_image())
        self.aws_file = self.awsfile.getFile()

    def test_index_html(self):
        """ Tests index_html method."""
        request = self.layer['request']
        response = request.RESPONSE
        self.aws_file.index_html(request, response)
        self.assertEqual(self.aws_file.absolute_url(),
                         request.RESPONSE.getHeader('location'))

    def test_absolute_url(self):
        """ Test url creation."""
        self.assertEqual('http://contentfiles.s3.amazonaws.com/' +
                         self.aws_file.source_id, self.aws_file.absolute_url())

    def test_remove_source(self):
        """ Test remove_source method."""
        self.aws_file.remove_source()
        aws_utility = getUtility(IAWSFileClientUtility)
        as3client = aws_utility.getFileClient()
        self.assert_(not as3client.get(self.aws_file.source_id))


def test_suite():
    suite = unittest2.TestSuite()
    suite.addTest(unittest2.makeSuite(AWSFileTestCase))
    return suite



from zope.interface import Interface

class IAWSField(Interface):
    """ Base interface for aws related fields."""

class IAWSFileField(IAWSField):
    """ Marker interface for AWSFileField """

class IAWSImageField(IAWSField):
    """ Marker interface for AWSImageField """




AWSCONF_SHEET = 'contentfiles2aws'
PROJECTNAME = 'collective.contentfiles2aws'

ADD_PERMISSIONS = {
    'AWSFile':'collective.contentfiles2aws: Add AWSFile',
    'AWSImage':'collective.contentfiles2aws: Add AWSImage',
}

SKIP_SOURCE_REMOVE_FLAG = 'collective.contentfiles2aws.skip_aws_source_remove'
ABORT_TRANSACTION_FLAG = 'collective.contentfiles2aws.abort_transaction'



from zope.interface import implements

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import PrimaryFieldMarshaller

from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.image import ATImage
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.ATContentTypes import ATCTMessageFactory as _

from Products.validation.config import validation
from Products.validation.validators.SupplValidators import MaxSizeValidator
from Products.validation import V_REQUIRED

from collective.contentfiles2aws.config import PROJECTNAME
from collective.contentfiles2aws.storage import AWSStorage
from collective.contentfiles2aws.fields import AWSImageField
from collective.contentfiles2aws.widgets import AWSImageWidget
from collective.contentfiles2aws.content.interfaces import IAWSImage


validation.register(MaxSizeValidator('checkImageMaxSize',
                                     maxsize=zconf.ATImage.max_file_size))


AWSImageSchema = ATContentTypeSchema.copy() + Schema((
    AWSImageField('image',
                  required=True,
                  primary=True,
                  languageIndependent=True,
                  storage = AWSStorage(),
                  swallowResizeExceptions = zconf.swallowImageResizeExceptions.enable,
                  pil_quality = zconf.pil_config.quality,
                  pil_resize_algo = zconf.pil_config.resize_algo,
                  max_size = zconf.ATImage.max_image_dimension,
                  sizes= {'large'   : (768, 768),
                          'preview' : (400, 400),
                          'mini'    : (200, 200),
                          'thumb'   : (128, 128),
                          'tile'    :  (64, 64),
                          'icon'    :  (32, 32),
                          'listing' :  (16, 16),
                         },
                  validators = (('isNonEmptyFile', V_REQUIRED),
                                ('checkImageMaxSize', V_REQUIRED)),
                  widget = AWSImageWidget(
                           description = '',
                           label= _(u'label_image', default=u'Image'),
                           show_content_type = False,)),

                  ), marshall=PrimaryFieldMarshaller()
                  )

AWSImageSchema['title'].required = False
finalizeATCTSchema(AWSImageSchema)


class AWSImage(ATImage):
    """An image, which can be referenced in documents or 
       displayed in an album.

    """

    schema         =  AWSImageSchema

    portal_type    = 'AWSImage'
    archetype_name = 'AWSImage'

    implements(IAWSImage)

registerATCT(AWSImage, PROJECTNAME)



from time import time
from md5 import md5
from random import random
from os.path import splitext

from Acquisition import aq_base
from OFS.Image import File, Image
from AccessControl import ClassSecurityInfo

from zope.component import getUtility

from Products.CMFPlone.utils import safe_unicode
from Products.statusmessages.interfaces import IStatusMessage
from Products.Archetypes.Storage import _marker
from Products.Archetypes.Storage.annotation import AnnotationStorage
from Products.Archetypes.interfaces import IReferenceable
from plone.i18n.normalizer.interfaces import IIDNormalizer

from collective.contentfiles2aws.awsfile import AWSFile
from collective.contentfiles2aws.interfaces import IAWSFileClientUtility
from collective.contentfiles2aws.client.fsclient import FileClientStoreError
from collective.contentfiles2aws.client.fsclient import FileClientRemoveError


class AWSStorage(AnnotationStorage):
    """ Archetype storage that stores data to amazon s3 service. """

    security = ClassSecurityInfo()

    def make_prefix(self):
        data = str(time() * 1000L) + str(random() * 100000000000000000L)
        return md5(data).hexdigest()[-7:]

    def getNormalizedName(self, filename):
        if filename:
            normalizer = getUtility(IIDNormalizer)
            return ".".join([normalizer.normalize(safe_unicode(n))
                            for n in splitext(filename)])

    def getSourceId(self, name, filename, instance, fresh=False):
        sid = ''
        if IReferenceable.providedBy(instance):
            sid = sid + instance.UID()
        sid = "%s_%s" % (sid, self.make_prefix())
        sid = "%s_%s" % (sid, name)
        fname = self.getNormalizedName(filename)
        if fname:
            sid = '%s_%s' % (sid, fname)
        return sid

    def update_source(self, file_, data, instance,
                      filename, content_type, width, height):
        aws_utility = getUtility(IAWSFileClientUtility)
        as3client = aws_utility.getFileClient()
        if file_.source_id:
            as3client.delete(file_.source_id)

        source_id = self.getSourceId(file_.id(), filename,
                                     instance, fresh=True)
        as3client.put(source_id, data, mimetype=content_type,
                      original_name=filename)
        setattr(file_, 'source_id', source_id)
        setattr(file_, 'size', len(data))
        setattr(file_, 'filename', filename)
        setattr(file_, 'content_type', content_type)

        if width and height:
            file_.width = width
            file_.height = height

    def _do_migrate(self, file_, instance, data=None, filename='',
                    content_type='', width='', height=''):
        if not data:
            data = file_.data

        if not filename:
            filename = getattr(file_, 'filename')

        if not content_type:
            content_type = getattr(file_, 'content_type')

        if not width:
            width = getattr(file_, 'width', '')

        if not height:
            height = getattr(file_, 'height', '')

        new_file = AWSFile(file_.id(), size=len(data),
                           filename=filename, content_type=content_type)
        try:
            self.update_source(new_file, data, instance,
                               filename, content_type, width, height)
            return new_file
        except (FileClientRemoveError, FileClientStoreError):
            # notify user???
            return file_

    def _migration(self, name, instance, **kwargs):
        """Migrates data from the original storage

        """
        value = getattr(aq_base(instance), name, _marker)
        if value is _marker:
                raise AttributeError(name)
        delattr(instance, name)

        # we use set method from base class to prevent migration to aws
        AnnotationStorage.set(self, name, instance, value, **kwargs)
        return value

    security.declarePrivate('get')
    def get(self, name, instance, **kwargs):
        aws_utility = getUtility(IAWSFileClientUtility)
        if not aws_utility.active():
            return AnnotationStorage.get(self, name, instance, **kwargs)

        file_ = AnnotationStorage.get(self, name, instance, **kwargs)
        request = instance.REQUEST
        if not isinstance(request, type('')) and \
                request.get('%s_migrate' % name, '') or \
                ('migrate' in kwargs and kwargs['migrate']):
            # check if object is already migrated
            if isinstance(file_, AWSFile):
                return file_
            try:
                new_file_ = self._do_migrate(file_, instance)
            except (FileClientRemoveError, FileClientStoreError):
                return file_

            AnnotationStorage.set(self, name, instance, new_file_, **kwargs)
            return new_file_
        else:
            return file_

    security.declarePrivate('set')
    def set(self, name, instance, value, **kwargs):
        """Set a value under the key 'name' for retrevial by/for
        instance."""

        # collect value info
        filename = getattr(value, 'filename', '')
        content_type = getattr(value, 'content_type', '')
        width = getattr(value, 'width', '')
        height = getattr(value, 'height', '')

        aws_utility = getUtility(IAWSFileClientUtility)
        if not aws_utility.active():
            if isinstance(value, AWSFile):
                # use default OFS.Image or OFS.File
                if width and height:
                    # we have image
                    value = Image(value.id(), '', str(value.data),
                                  content_type=content_type)
                else:
                    value = File(value.id(), '', str(value.data),
                                 content_type=content_type)
                setattr(value, 'filename', filename)
            AnnotationStorage.set(self, name, instance, value, **kwargs)
            return

        try:
            file_ = self.get(name, instance, **kwargs)
        except AttributeError:
            file_ = None

        if file_:
            if isinstance(file_, AWSFile):
                try:
                    self.update_source(file_, value.data, instance,
                                       filename, content_type, width, height)
                except (FileClientRemoveError, FileClientStoreError), e:
                    request = instance.REQUEST
                    IStatusMessage(request).addStatusMessage(
                        u"Couldn't update %s file to storage. %s" %
                        (safe_unicode(filename),
                         safe_unicode(e.message)), type='error')
                    if isinstance(value, AWSFile):
                        # use default OFS.Image or OFS.File
                        if width and height:
                            # we have image
                            value = Image(value.id(), '', str(value.data),
                                          content_type=content_type)
                        else:
                            value = File(value.id(), '', str(value.data),
                                         content_type=content_type)
                        setattr(value, 'filename', filename)
                    AnnotationStorage.set(self, name, instance,
                                          value, **kwargs)
                else:
                    # clean up data after update
                    setattr(file_, 'data', '')
            else:
                try:
                    file_ = self._do_migrate(file_, instance,
                                             data=value.data,
                                             filename=filename,
                                             content_type=content_type,
                                             width=width,
                                             height=height)
                except (FileClientRemoveError, FileClientStoreError):
                    request = instance.REQUEST
                    IStatusMessage(request).addStatusMessage(
                        u"Couldn't update %s file to storage. %s" %
                        (safe_unicode(filename),
                         safe_unicode(e.message)), type='error')
                    AnnotationStorage.set(self, name, instance,
                                          value, **kwargs)
                else:
                    AnnotationStorage.set(self, name, instance,
                                          file_, **kwargs)
        else:
            if value.size:
                file_ = AWSFile(name)
                try:
                    self.update_source(file_, value.data, instance,
                                       filename, content_type, width, height)
                except (FileClientRemoveError, FileClientStoreError), e:
                    request = instance.REQUEST
                    IStatusMessage(request).addStatusMessage(
                        u"Couldn't update %s file to storage. %s" %
                        (safe_unicode(filename),
                         safe_unicode(e.message)), type='error')

                    AnnotationStorage.set(self, name, instance,
                                          value, **kwargs)
                else:
                    AnnotationStorage.set(self, name, instance,
                                          file_, **kwargs)
            else:
                AnnotationStorage.set(self, name, instance, file_, **kwargs)

    security.declarePrivate('unset')
    def unset(self, name, instance, **kwargs):
        aws_utility = getUtility(IAWSFileClientUtility)
        if not aws_utility.active():
            return AnnotationStorage.unset(self, name, instance, **kwargs)

        file_ = self.get(name, instance, **kwargs)
        if isinstance(file_, AWSFile):
            file_.remove_source()
        AnnotationStorage.unset(self, name, instance, **kwargs)



from zope.component import getUtility

from Products.Five.browser import BrowserView

from collective.contentfiles2aws.awsfile import AWSFile
from collective.contentfiles2aws.interfaces import IAWSFileClientUtility


class AWSUtilView(BrowserView):
    """ Helper view.  """

    def getFieldValue(self, instance, name, scale=None):
        field = instance.getField(name)
        if scale:
            return field.getScale(instance, scale=scale)

        accessor = field.getAccessor(instance)
        return accessor()

    def getUrlFromBrain(self, brain, name, scale=None):
        if scale:
            name = '%s_%s' % (name, scale)

        if hasattr(brain, 'aws_sources') and brain.aws_sources and \
           name in brain.aws_sources:
            sid = brain.aws_sources[name]
            aws_utility = getUtility(IAWSFileClientUtility)
            bucket_name = aws_utility.getBucketName()
            client = aws_utility.getFileClient()
            url = "http://%s.%s" % (bucket_name, client.connection.server)

            filename_prefix = aws_utility.getAWSFilenamePrefix()
            if filename_prefix:
                url = '%s/%s' % (url, filename_prefix)

            url = '%s/%s' % (url, sid)
        else:
            url = '%s/%s' % (brain.getURL(), name)
        return url

    def getUrlFromObject(self, instance, name, scale=None):
        value = self.getFieldValue(instance, name, scale=scale)
        if isinstance(value, AWSFile):
            url = value.absolute_url()
        else:
            url = '%s/%s' % (instance.absolute_url(), name)
            if scale:
                url = '%s_%s' % (url, scale)
        return url

    def get_file_url(self, instance, name='file', brain=True):
        """ Generates url for file stored in field. """

        return brain and self.getUrlFromBrain(instance, name) or \
            self.getUrlFromObject(instance, name)

    def get_image_url(self, instance, name='image', scale=None, brain=True):
        """ Generates url for file stored in field. """

        return brain and self.getUrlFromBrain(instance, name, scale=scale) or \
            self.getUrlFromObject(instance, name, scale=scale)



from zope.annotation import IAnnotations

from collective.contentfiles2aws import config
from collective.contentfiles2aws.awsfile import AWSFile
from collective.contentfiles2aws.interfaces import IAWSField

def mark_request(request, name, value):
    annotations = IAnnotations(request)
    annotations[name] = value

def abort_transaction(request):
    mark_request(request, config.ABORT_TRANSACTION_FLAG, 1)

def skip_source_remove(request):
    mark_request(request, config.SKIP_SOURCE_REMOVE_FLAG, 1)

def replace_source_uid(sid, uid):
    old_uid = sid.split('_')[0]
    return  sid.replace(old_uid, uid)

def getAWSFields(obj):
    """ Collect aws fields for provided object.

    Goes through object schema fields and collects
    all aws fields.

    :param obj: content object.

    """
    aws_fields = []
    for f in obj.schema.fields():
        if IAWSField.providedBy(f):
            accessor= f.getAccessor(obj)
            field_content = accessor()
            if isinstance(field_content, AWSFile):
                aws_fields.append((f, field_content))
    return aws_fields



from collective.contentfiles2aws.interfaces.utility import \
        IAWSFileClientUtility
from collective.contentfiles2aws.interfaces.fields import IAWSField
from collective.contentfiles2aws.interfaces.fields import IAWSFileField
from collective.contentfiles2aws.interfaces.fields import IAWSImageField



from plone.app.testing import PloneSandboxLayer
from plone.app.testing import PLONE_FIXTURE, PLONE_INTEGRATION_TESTING
from plone.app.testing import IntegrationTesting

from plone.testing import z2
from plone.testing import Layer


class AWSContentFilesFixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import collective.contentfiles2aws
        self.loadZCML(package=collective.contentfiles2aws)

        z2.installProduct(app, 'collective.contentfiles2aws')


    def setUpPloneSite(self, portal):
        self.applyProfile(portal, 'collective.contentfiles2aws:default')

AWS_CONTENT_FILES_FIXTURE = AWSContentFilesFixture()


class AWSContentFilesUnitTestsFixture(Layer):
    """Unit tests layer for choosehelp.content package"""
    pass

AWS_CONTENT_FILES_UNIT_TESTS_FIXTURE = AWSContentFilesUnitTestsFixture()


class AWSFixture(Layer):
    """ Simple fixture for amazon related tests. """

    def patchS3Connection(self):
        """ Patches S3 connection with mock connection."""

        import boto.s3.connection
        from collective.contentfiles2aws.tests.s3_service_mockup import \
                MockConnection

        boto.s3.connection.S3Connection = MockConnection

        # we need to reload our client module
        # to be able to use patched version.
        import collective.contentfiles2aws.client.awsclient
        reload(collective.contentfiles2aws.client.awsclient)

    def setUp(self):
        """ Sets up amazon fixture. """
        self.patchS3Connection()

AWS_FIXTURE = AWSFixture()

AWS_CONTENT_FILES_INTEGRATION_TESTING = IntegrationTesting(bases=(
    PLONE_INTEGRATION_TESTING, AWS_FIXTURE, AWS_CONTENT_FILES_FIXTURE),
    name="AWSContentFiles:Integration")



import os
import unittest2

from OFS.Image import Image

from collective.contentfiles2aws.awsfile import AWSFile
from collective.contentfiles2aws.testing import \
    AWS_CONTENT_FILES_INTEGRATION_TESTING


class AWSImageFieldTestCase(unittest2.TestCase):
    """
    """
    layer = AWS_CONTENT_FILES_INTEGRATION_TESTING

    def _get_image(self):
        dir_name = os.path.dirname(os.path.abspath(__file__))
        return open('%s/data/image.gif' % dir_name, 'rb')

    def setUp(self):
        self.portal = self.layer['portal']
        fid = self.portal.invokeFactory('AWSImage', 'awsimage')
        self.awsimage = getattr(self.portal, fid)
        self.awsimage.update(image=self._get_image())
        self.conf_sheet = self.portal.portal_properties.contentfiles2aws

    def test_wrapValue(self):
        field = self.awsimage.schema['image']
        image = field._wrapValue(self.awsimage, '')
        self.assert_(isinstance(image, Image))

        self.assert_(isinstance(field._wrapValue(self.awsimage, image), Image))
        awsfile = AWSFile('awsfile')
        self.assert_(isinstance(field._wrapValue(self.awsimage, awsfile),
                                AWSFile))

    def test_get(self):
        field = self.awsimage.schema['image']
        image = field.get(self.awsimage)
        self.assert_(isinstance(image, Image))

        self.conf_sheet._updateProperty('USE_AWS', True)
        self.awsimage.REQUEST['image_migrate'] = True
        image = field.get(self.awsimage)
        self.assert_(isinstance(image, AWSFile))
        for n in field.getAvailableSizes(self.awsimage).keys():
            scale = field.getScale(self.awsimage, scale=n)
            self.assert_(isinstance(scale, AWSFile))

    def test_set(self):
        self.conf_sheet._updateProperty('AWS_BUCKET_NAME', 'contentfiles')
        self.conf_sheet._updateProperty('USE_AWS', True)
        fid = self.portal.invokeFactory('AWSImage', 'awsimage2')
        awsimage2 = getattr(self.portal, fid)

        awsimage2.schema['image'].set(awsimage2, self._get_image())
        self.assert_(awsimage2.schema['image'].url(
            awsimage2).startswith('http://contentfiles.s3.amazonaws.com/'))

        image_field = awsimage2.schema['image']
        for n, s in image_field.getAvailableSizes(awsimage2).items():
            storage = image_field.getStorage(awsimage2)
            scale = storage.get('image_' + n, awsimage2)
            self.assertEqual('AWS File', scale.meta_type)

    def test_createScales(self):
        self.conf_sheet._updateProperty('USE_AWS', True)
        field = self.awsimage.schema['image']
        field.sizes['new_scale'] = (100, 100)
        field.createScales(self.awsimage)
        new_scale = field.getScale(self.awsimage, scale='new_scale')
        self.assert_(isinstance(new_scale, AWSFile))

    def test_tag(self):
        self.conf_sheet._updateProperty('USE_AWS', True)
        self.conf_sheet._updateProperty('AWS_BUCKET_NAME', 'contentfiles')
        field = self.awsimage.schema['image']

        self.assertEqual(field.tag(self.awsimage, scale='thumb'),
                        ('<img src="http://nohost/plone/awsimage/image_thumb"'
                         ' alt="" title="" height="16" width="16" />'))
        self.awsimage.REQUEST['image_migrate'] = True
        field.get(self.awsimage)
        tag = field.tag(self.awsimage, scale='thumb')
        self.assert_(
            tag.startswith('<img src="http://contentfiles.s3.amazonaws.com/'))
        self.assert_(tag.endswith(('image_thumb_image.gif" alt="" title=""'
                                   ' height="16" width="16" />')))


def test_suite():
    suite = unittest2.TestSuite()
    suite.addTest(unittest2.makeSuite(AWSImageFieldTestCase))
    return suite



from setuptools import setup, find_packages
import os

version = '1.2.1'

setup(name='collective.contentfiles2aws',
      version=version,
      description="Allows to store files and images on amazon s3 service.",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Programming Language :: Python",
        ],
      keywords='Plone AWS',
      author='Taras Melnychuk',
      author_email='melnychuktaras@gmail.com',
      url='https://github.com/martinschoel/collective.contentfiles2aws.git',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'boto',
      ],
      extras_require = {'tests': ['plone.app.testing',]},
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



import os
import unittest2

from Products.CMFCore.utils import getToolByName
from ZODB.POSException import ConflictError

from collective.contentfiles2aws.testing import \
    AWS_CONTENT_FILES_INTEGRATION_TESTING

_marker = []

try:
    import PIL.Image
except ImportError:
    # no PIL, no scaled versions!
    log("Warning: no Python Imaging Libraries (PIL) found."+\
        "Archetypes based ImageField's don't scale if neccessary.")
    HAS_PIL=False
    PIL_ALGO = None
else:
    HAS_PIL=True
    PIL_ALGO = PIL.Image.ANTIALIAS

def createScales(self, instance, value=_marker):
    """creates the scales and save them
    """
    #import pdb; pdb.set_trace()
    sizes = self.getAvailableSizes(instance)
    if not HAS_PIL or not sizes:
        return
    # get data from the original size if value is None
    if value is _marker:
        img = self.getRaw(instance)
        if not img:
            return
        data = str(img.data)
    else:
        data = value

    # empty string - stop rescaling because PIL fails on an empty string
    if not data:
        return

    filename = self.getFilename(instance)

    for n, size in sizes.items():
        if size == (0,0):
            continue
        w, h = size
        id = self.getName() + "_" + n
        __traceback_info__ = (self, instance, id, w, h)
        try:
            imgdata, format = self.scale(data, w, h)
        except (ConflictError, KeyboardInterrupt):
            raise
        except:
            if not self.swallowResizeExceptions:
                raise
            else:
                log_exc()
                # scaling failed, don't create a scaled version
                continue

        mimetype = 'image/%s' % format.lower()
        if n == 'tile':
            pp = getToolByName(instance, 'portal_properties')
            pp.contentfiles2aws._updateProperty('USE_AWS', False)
        image = self._make_image(id, title=self.getName(), file=imgdata,
                                 content_type=mimetype, instance=instance)
        # nice filename: filename_sizename.ext
        #fname = "%s_%s%s" % (filename, n, ext)
        #image.filename = fname
        image.filename = filename
        try:
            delattr(image, 'title')
        except (KeyError, AttributeError):
            pass
        # manually use storage
        self.getStorage(instance).set(id, instance, image,
                                      mimetype=mimetype, filename=filename)
        if n == 'tile':
            pp = getToolByName(instance, 'portal_properties')
            pp.contentfiles2aws._updateProperty('USE_AWS', True)

class AWSIndexersTestCase(unittest2.TestCase):
    """ AWSINdexers test case. """

    layer =  AWS_CONTENT_FILES_INTEGRATION_TESTING

    def _get_image(self):
        dir_name = os.path.dirname(os.path.abspath(__file__))
        return open('%s/data/image.gif' % dir_name, 'rb')

    def patch_scale_creation(self):
        from Products.Archetypes.Field import ImageField
        ImageField.createScales= createScales

    def create_test_image(self):
        self.conf_sheet._updateProperty('AWS_BUCKET_NAME', 'contentfiles')
        self.conf_sheet._updateProperty('USE_AWS', True)

        fid = self.portal.invokeFactory('AWSImage', 'aws_image')
        aws_image = getattr(self.portal, fid)
        aws_image.update(image=self._get_image())

    def setUp(self):
        self.portal = self.layer['portal']
        self.conf_sheet=self.portal.portal_properties.contentfiles2aws

    def test_aws_sources(self):
        """ This specific test was made to cover issue #1444.

        This test covers fix for case when not all image scales was
        created as amazon images due to some issues with network or
        access to amazon service.
        """

        self.patch_scale_creation()
        self.create_test_image()

        obj = getattr(self.portal, 'aws_image')

        from collective.contentfiles2aws.catalog import aws_sources
        indexer = aws_sources(obj)
        self.assert_('image_tile' not in indexer())


def test_suite():
    suite = unittest2.TestSuite()
    suite.addTest(unittest2.makeSuite(AWSIndexersTestCase))
    return suite



from zope.interface import implements

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import PrimaryFieldMarshaller

from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.file import ATFile

from Products.ATContentTypes import ATCTMessageFactory as _
from Products.validation import V_REQUIRED

from collective.contentfiles2aws.config import PROJECTNAME
from collective.contentfiles2aws.storage import AWSStorage
from collective.contentfiles2aws.fields import AWSFileField
from collective.contentfiles2aws.widgets import AWSFileWidget
from collective.contentfiles2aws.content.interfaces import IAWSFile


__docformat__ = 'restructuredtext'


AWSFileSchema = ATContentTypeSchema.copy() + Schema((
    AWSFileField('file',
              required=True,
              primary=True,
              searchable=False,
              languageIndependent=True,
              storage = AWSStorage(),
              validators = (('isNonEmptyFile', V_REQUIRED),
                             ('checkFileMaxSize', V_REQUIRED)),
              widget = AWSFileWidget(
                           description = '',
                           label=_(u'label_file', default=u'File'),
                           show_content_type = False,)),
    ), marshall=PrimaryFieldMarshaller()
    )

AWSFileSchema['title'].required = False
finalizeATCTSchema(AWSFileSchema)


class AWSFile(ATFile):
    """An external file uploaded to amazon."""

    schema         =  AWSFileSchema
    portal_type    = 'AWSFile'
    archetype_name = 'AWSFile'

    implements(IAWSFile)

registerATCT(AWSFile, PROJECTNAME)



import os
import unittest2

from zope.component import getMultiAdapter

from Products.CMFCore.utils import getToolByName
from ZODB.POSException import ConflictError

from collective.contentfiles2aws.testing import \
    AWS_CONTENT_FILES_INTEGRATION_TESTING

from collective.contentfiles2aws.tests.test_catalog import createScales


class AWSIndexersTestCase(unittest2.TestCase):
    """ AWSINdexers test case. """

    layer =  AWS_CONTENT_FILES_INTEGRATION_TESTING

    def _get_image(self):
        dir_name = os.path.dirname(os.path.abspath(__file__))
        return open('%s/data/image.gif' % dir_name, 'rb')

    def patch_scale_creation(self):
        from Products.Archetypes.Field import ImageField
        ImageField.createScales= createScales

    def create_test_image(self):
        self.conf_sheet._updateProperty('AWS_BUCKET_NAME', 'contentfiles')
        self.conf_sheet._updateProperty('USE_AWS', True)

        fid = self.portal.invokeFactory('AWSImage', 'aws_image')
        aws_image = getattr(self.portal, fid)
        aws_image.update(image=self._get_image())

    def setUp(self):
        self.portal = self.layer['portal']
        self.conf_sheet=self.portal.portal_properties.contentfiles2aws

    def test_aws_image_url(self):
        """ This specific test was made to cover issue #1444.

        This test covers fix for case when not all image scales was
        created as amazon images due to some issues with network or
        access to amazon service.
        """

        self.patch_scale_creation()
        self.create_test_image()

        portal = self.layer['portal']
        request = self.layer['request']

        obj_brain = portal.portal_catalog(id='aws_image')[0]
        aws_image_url = getMultiAdapter((portal, request),
                                        name = 'aws_image_url')
        self.assertEqual(aws_image_url(obj_brain, scale='tile'),
                         'http://nohost/plone/aws_image/image_tile')


def test_suite():
    suite = unittest2.TestSuite()
    suite.addTest(unittest2.makeSuite(AWSIndexersTestCase))
    return suite



import transaction

from Acquisition import aq_inner
from Acquisition import aq_parent
from zope.annotation import IAnnotations
from zope.component import getUtility

from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage

from collective.contentfiles2aws import utils
from collective.contentfiles2aws import MFactory as _
from collective.contentfiles2aws.interfaces import IAWSFileClientUtility
from collective.contentfiles2aws.interfaces import IAWSImageField
from collective.contentfiles2aws.client.fsclient import FileClientRemoveError
from collective.contentfiles2aws.client.fsclient import FileClientCopyError
from collective.contentfiles2aws.config import ABORT_TRANSACTION_FLAG
from collective.contentfiles2aws.config import SKIP_SOURCE_REMOVE_FLAG

class AWSSourceRemoveError(Exception):
    pass

class AWSSourceCopyError(Exception):
    pass

def remove_source(obj):
    """ Remove file data from amazon.

    :param obj: content object.

    """
    obj_fields = utils.getAWSFields(obj)
    for field, value in obj_fields:
       aws_utility = getUtility(IAWSFileClientUtility)
       if not aws_utility.active():
           raise AWSSourceRemoveError("Could not delete remote source. "
                                      "To be able to delete object properly, "
                                      "please activate AWS storage")
       as3client = aws_utility.getFileClient()
       if hasattr(value, 'source_id') and value.source_id:
           try:
               as3client.delete(value.source_id)
               if IAWSImageField.providedBy(field):
                   field.removeScales(obj)
           except FileClientRemoveError, e:
               raise AWSSourceRemoveError(e.message)

def clone_source(obj):
    """ Creates copy of file data on amazon.

    :param obj: content object.
    :param remove_origin: if set to true origin source will be removed.
    :type remove_origin: boolean

    """

    def copy_source(aws_file, obj):
        old_sid = aws_file.source_id
        new_sid = utils.replace_source_uid(old_sid, obj.UID())

        if old_sid == new_sid:
            # source already cloned
            return

        as3client = aws_utility.getFileClient()
        as3client.copy_source(old_sid, new_sid)
        aws_file.source_id = new_sid

    obj_fields = utils.getAWSFields(obj)
    for field, value in obj_fields:
        aws_utility = getUtility(IAWSFileClientUtility)
        if not aws_utility.active():
            raise AWSSourceCopyError("Could not copy remote source. "
                                     "To be able to copy object properly, "
                                     "please activate AWS storage")
        if hasattr(value, 'source_id') and \
                value.source_id:
            try:
                copy_source(value, obj)
                if IAWSImageField.providedBy(field):
                    for n in field.getAvailableSizes(obj).keys():
                        copy_source(field.getScale(obj, scale=n), obj)
            except (FileClientCopyError, FileClientRemoveError), e:
                raise AWSSourceCopyError(e.message)

def before_file_remove(obj, event):
    request = getattr(obj, 'REQUEST', '')
    if request and hasattr(request, 'ACTUAL_URL') and \
       request.ACTUAL_URL.endswith('delete_confirmation') and \
            request.get('REQUEST_METHOD') == 'GET':
                # delete event is fired by link integrity check, skip it.
                return

    # skip source remove if we have appropriate flag set in request
    annotations = IAnnotations(request)
    if annotations and annotations.has_key(SKIP_SOURCE_REMOVE_FLAG) and \
            annotations[SKIP_SOURCE_REMOVE_FLAG]:
        return

    try:
        remove_source(obj)
    except AWSSourceRemoveError, e:
        IStatusMessage(request).addStatusMessage(_(e), type='error')
        utils.abort_transaction(request)

def abort_remove(obj, event):
    annotations = IAnnotations(obj.REQUEST)
    if annotations and annotations.has_key(ABORT_TRANSACTION_FLAG) and \
            annotations[ABORT_TRANSACTION_FLAG]:
        transaction.abort()

def object_cloned(obj, event):

    def cleanup(obj):
        catalog = getToolByName(obj, 'portal_catalog')
        container = aq_parent(aq_inner(obj))
        # we have to skip source remove to be able to remove object.
        utils.skip_source_remove(obj.REQUEST)
        catalog.uncatalog_object('/'.join(obj.getPhysicalPath()))
        for brain in catalog(path={'depth': 1,
            'query': '/'.join(obj.getPhysicalPath())}):
            catalog.uncatalog_object(brain.getPath())
        container.manage_delObjects(ids=[obj.getId()])

    request = obj.REQUEST
    try:
        clone_source(obj)
    except AWSSourceCopyError, e:
        cleanup(obj)
        if aq_parent(aq_inner(obj)) is event.object:
            # clean up parent object
            cleanup(event.object)
        IStatusMessage(request).addStatusMessage(_(e), type='error')
        request.RESPONSE.redirect(request.get('HTTP_REFERER'))
